eval 'exec perl -w -S $0 ${1+"$@"}'
    if 0;

################################################################################
#
# $RCSfile: iowlcd_amarok.pl $
#
#
# This program is Copyright (C) 02/2006 Matthias Prinke
# <matthias.prinke@surfeu.de> and covered by GNU's GPL.
# In particular, this program is free software and comes WITHOUT
# ANY WARRANTY.
#
# $Author: mp $
#
# $Date: 2006/02/10 $
#
# $Revision: 0.0.2 $
#
################################################################################

package iowlcd_amarok;

use strict;
use warnings;
use DCOP::Amarok::Player;
use IO::Socket;
use threads;
use threads::shared;
# The Tk-Modules are loaded by installed() if available
#use Tk;
#use Tk::NumEntry;

our $VERSION = '0.0.2';

#### BEGIN CONFIG
# Configuration Options
## iowlircemu
my $priority  : shared = 10;
my $lcd_width : shared = 20;
my $user               = "$ENV{USER}";
### END CONFIG

=head1 NAME

iowlcd_amarok

=head1 DESCRIPTION

This program is receives status information via DCOP
from amaroK, and sends it via a UNIX domain socket
to iowlircemu which then shows it on the LCD.

=cut

my $have_tk = 1;
my @mod_check = qw(Tk Tk::NumEntry);
my $amarok = DCOP::Amarok::Player->new( user => "$user" ) or die "Could'nt attach to DCOP: $!\n";
my $status : shared = ( $amarok->status() > 0 ) ? 1 : 0;
my $cfgfile = "iowlcd_amarok.cfg";
my @lcd_width_opts = (" 8", "16", "20", "40");


# try to read config file
if (open (CFGFILE, "$cfgfile")) {
  while (<CFGFILE>) {
    # priority = <value>
    if (/^\s*priority\s*=\s*(\S+)/) {
      if (($1 > -1) && ($1 < 256)) {
        $priority = $1;
      }
    } elsif (/^\s*lcd_width\s*=\s*(\S+)/) {      
      if (($1 > 7) && ($1 < 81)) {
        $lcd_width = $1;
      }
    }
  }
  close (CFGFILE);
}


# Check if Perl Modules Configuration GUI are available
for (@mod_check) {
  if ( not(installed("$_")) ) {
    print "iowlcd_amarok: $_ is not installed.\n";
    print "iowlcd_amarok: Configuration GUI will not be available.\n";
    $have_tk = 0;
    last;
  }
}


# create unix socket connection to iowlircemu
socket(CLIENT, PF_UNIX, SOCK_STREAM, 0);
connect(CLIENT, sockaddr_un("/dev/iowlcd"))
  or die "iowlcd_amarok: Can't connect to /dev/iowlcd: $!";
close(CLIENT);

# time in seconds between LCD updates
my $sleep  : shared = 1;

# catch TERM signal
$SIG{TERM} = \&bye;

# create thread for LCD update
my $thread = threads->create(\&update_disp);
$thread->detach();


# receive some status info via STDIN from amaroK and handle it
while(1) {
	$_ = <STDIN>;

        if ((/configure/) && ($have_tk == 1)) {
	  print "\niowlcd_amarok: configure\n";
	  my $mw = MainWindow->new();
          $mw->resizable(0, 0);
	  $mw->title( "Config" );
	  $mw->Label(-text => "iowlcd_amarok", -justify => 'left')
	           ->pack(-side => 'top');
	  $mw->Label(-text => "Display Priority:", -justify => 'left')
	           ->pack(-side => 'top');
	  my $entry = $mw->NumEntry(-justify => 'right', -validate => 'key',
		  -minvalue => 0, -maxvalue => 255, -value => $priority)
		  ->pack(-side => 'top');
	  $mw->Label(-text => "LCD Width (Chars):", -justify => 'left')
	           ->pack(-side => 'top');	
          $mw->Optionmenu(-textvariable => \$lcd_width,
	           -options => \@lcd_width_opts)
                   ->pack(-side => 'top');
	  $mw->Button(-text => "Cancel",
	           -command => sub {
		     $priority = $entry->get();
	             $mw->destroy;
		     $mw = undef})
		  ->pack(-fill => 'x', -side => 'left');
	  $mw->Button(-text => "OK", -default => 'active', -takefocus => '1',
	           -padx => 25,
	           -command => sub {
		     $priority = $entry->get();
	             $mw->destroy;
		     write_config($priority, $lcd_width);
		     $mw = undef})
		  ->pack(-fill => 'x',-side => 'right');
	  &MainLoop;
	  
	} elsif ( /trackChange/ ) {
          printf("\n%s-%s\n", $amarok->artist(), $amarok->title());
	} elsif ( /engineStateChange/ ) {
	  if( /playing/ ) {
	    $status = 1;
	    printf("\niowlcd_amarok: Playing %s\n",$amarok->title());
	  } elsif ( /pause/ ) {
	    $status = 1;
	    print "\niowlcd_amarok: pause\n";
	  } else {
	    $status = 0;
	    print "\niowlcd_amarok: stopped\n";
	  }
	} elsif( /volumeChange/ ){
	  m/: (\d+)/;
	  print "\niowlcd_amarok: Vol: $1\n";
	}
	exit if(/exit/); #for debugging
	&bye() if(/kill/); # first kill, then exit... after thread has exited
}


# update display
sub update_disp {
  while(1) {    
    if ($status) {
      printf("\riowlcd_amarok: %s/%s",
        $amarok->elapsed(), $amarok->totaltime());

      # create unix socket connection to iowlircemu
      socket(CLIENT, PF_UNIX, SOCK_STREAM, 0);
      connect(CLIENT, sockaddr_un("/dev/iowlcd"))
        or die "Can't connect to /dev/iowlcd: $!";

      # print parts of first line to buffer
      my $line1 = sprintf("%-2d %s/%s",
	  $amarok->track(),
	  $amarok->elapsed(),
	  $amarok->totaltime()
        );
      
      
      if ($lcd_width == 8) {
        # this is for the really poor...
	printf(CLIENT "\\p%d\\%-2d %s\\n%s-%s\\n",
	  $priority,
	  $amarok->track(),
	  $amarok->elapsed(),
	  $amarok->artist(),
	  $amarok->title());  
      } elsif ($lcd_width == 16) {
        # still limited - leave out RND info

	# padding to align volume to the right
	my $pad = $lcd_width - length($line1) - 3;
	$pad = ($pad < 0) ? 0 : $pad;

	printf(CLIENT "\\p%d\\%-2d %s/%s%s%3d\\n%s-%s\\n",
	  $priority,
	  $amarok->track(),
	  $amarok->elapsed(),
	  $amarok->totaltime(),
          ' ' x $pad,
	  $amarok->vol(),
	  $amarok->artist(),
	  $amarok->title());
      } else {
        # full featured

	# padding to align volume to the right
	my $pad = $lcd_width - length($line1) - 8;
	$pad = ($pad < 0) ? 0 : $pad;

	printf(CLIENT "\\p%d\\%s %s %s%3d\\n%s-%s\\n",
	  $priority,
          $line1,
	  ($amarok->getRandom() eq "true") ? "RND" : "   ",
          ' ' x $pad,
	  $amarok->vol(),
	  $amarok->artist(),
	  $amarok->title());
      }
    }
    close(CLIENT);
    	
    if($status == -1) {
      return;
    }
    sleep $sleep;
  }
  system( "kdialog", "--sorry", "Im not supposed to appear..... Diing" );
}


# write config file
sub write_config {
  my $prio      = shift;
  my $lcd_width = shift;
  
  if (open (CFGFILE, ">$cfgfile")) {
    printf(CFGFILE "priority  = %u\n", $prio);
    printf(CFGFILE "lcd_width = %u\n", $lcd_width);
    close(CFGFILE);
  }
}


# Perl Module availability checking subroutine
sub installed {
  my $module = $_;

  my $check = 0;

  eval "use $module";
  if ($@) {
    $check = 0;
  } else {
    $check = 1; # non-zero equals installed.
    my $version = 0;
    eval "\$version = \$$module\::VERSION";
    $check = $version if (!$@); # Set check to version number if it exists
  }
}


# end thread disp_time and clear LCD
sub bye {
  $status = -1;
  sleep $sleep;

  # create unix socket connection to iowlircemu
  socket(CLIENT, PF_UNIX, SOCK_STREAM, 0);
  connect(CLIENT, sockaddr_un("/dev/iowlcd"));
  printf(CLIENT "\\n\\n\\n\\n");
  close(CLIENT);
  exit 0;
}

__END__

=head1 AUTHOR

M. Prinke E<lt>matthias.prinke@surfeu.deE<gt>
based on amaroklcd by Juan C. Muller E<lt>jcmuller@gmail.comE<gt>

=head1 COPYRIGHT (COPYLEFT)

2006, M. Prinke

=head1 LICENSE

This programs is licensed under the terms of the GPL. 

=cut
